.setcpu "6502X"

.include "ns_nes.s" ; handy macros and defines
.include "common.s" ; handy routines

.segment "INESHDR"
  .byt "NES", $1A
  .byt 1 ; PRG in 16kb units
  .byt 0 ; CHR in 8kb units
  .byt 0 ; vertical mirroring
  .byt 0
.include "charmap.s"
.segment "VECTORS"
Vectors:
  .addr nmi, reset, irq
.segment "CODE"

.include "main.s"
.include "memory.s"
.include "musicseq.h"
.include "musicseq.s"
.include "sound.s"
.include "music.s"

.macro PositionPrint XP, YP, Text
  PositionXY 0,  XP,  YP
  jsr PutStringImmediate	
  .byt Text,0
.endmacro

.proc reset
  lda #0		; Turn off PPU
  sta PPUCTRL
  sta PPUMASK
  sei
  ldx #$FF	; Set up stack pointer
  txs		; Wait for PPU to stabilize

: lda PPUSTATUS
  bpl :-
: lda PPUSTATUS
  bpl :-

  lda #0
  ldx #0
: sta $000,x
  sta $100,x 
  sta $300,x 
  sta $400,x 
  sta $500,x 
  sta $600,x 
  sta $700,x 
  inx
  bne :-
  sta OAM_DMA

  jsr init_sound

  lda #<SoundData1
  sta psg_sound_table+0
  lda #>SoundData1
  sta psg_sound_table+1
  lda #25
  sta Sound1Len
  sta Sound2Len

  lda #<SoundData2
  sta psg_sound_table+4
  lda #>SoundData2
  sta psg_sound_table+5

  lda #1
  sta r_seed

  lda #0
  sta PPUMASK

  jsr ClearName

  PositionXY 0,  2,  2
  jsr PutStringImmediate	
  .byt "SFX 1   SFX 2",0

  PositionXY    0, 17,  2
  lda #$30
  sta PPUDATA
  PositionXY    0, 25,  2
  lda #$30
  sta PPUDATA

  PositionPrint 17,  4, "PLAY"
  PositionPrint 17,  5, "PLAY 1"
  PositionPrint 17,  6, "PLAY 2"
  PositionPrint 17,  7, "SFX CHANNEL"
  PositionPrint 17,  8, "SWAP SFX"
  PositionPrint 17,  9, "COPY SFX"
  PositionPrint 17, 10, "CLEAR SFX"
  PositionPrint 17, 11, "CLONE ROW"
  PositionPrint 17, 12, "ZERO ROW"
  PositionPrint 17, 13, "DELETE ROW"
  PositionPrint 17, 14, "EXIT TO MENU"

  PositionPrint 17, 18, "DPAD: MOVE"
  PositionPrint 17, 19, "A+DPAD: EDIT"
  PositionPrint 17, 20, "START: PLAY"
  PositionPrint 17, 21, "B: MENU"

  lda #$3F
  sta PPUADDR
  lda #$00
  sta PPUADDR

  ldx #8
: lda #$0f ; black background rather than 2a for high contrast
  sta PPUDATA
  lda #$0f
  sta PPUDATA
  lda #$01
  sta PPUDATA
  lda #$30
  sta PPUDATA
  dex
  bne :-

  .scope
ChrPointer = 10
TilesLeft = 12
  lda #$00
  sta PPUADDR
  sta PPUADDR
  lda #<CHR
  sta ChrPointer+0
  lda #>CHR
  sta ChrPointer+1
  lda #48
  sta TilesLeft

CharLoop:
  .repeat 2
    ldy #0
  : lda (ChrPointer),y
    sta PPUDATA
    iny
    cpy #8
    bne :-
  .endrep

  lda ChrPointer+0
  add #8
  sta ChrPointer+0
  bcc :+
    inc ChrPointer+1
  :

  dec TilesLeft
  bne CharLoop

  .endscope

; make two solid blocks at the end of the font
  ldx #16+8
  lda #255
: sta PPUDATA
  dex
  bne :-
; -----------
  ldx #16
  lda #0
: sta PPUDATA
  dex
  bne :-
; -----------
  ldx #8
  lda #255
: sta PPUDATA
  dex
  bne :-

  jsr init_sound

  lda #VBLANK_NMI | NT_2000 | OBJ_8X8 | BG_0000 | OBJ_0000
  sta PPUCTRL

  ldx #0
: lda InitialSFX,x
  sta SoundData1,x
  inx
  cpx #InitialSFXEnd-InitialSFX
  bne :-

  jsr UpdateAllRows

  lda #0
  sta PPUSCROLL
  sta PPUSCROLL

  lda #BG_ON + OBJ_ON
  sta PPUMASK

  sei
  lda retraces
  sta r_seed 

  jmp MainLoop

InitialSFX:
  .byt $4f, $20, $4f, $21
  .byt $4f, $22, $4f, $23
  .byt $4f, $24, $4f, $25
  .byt $4f, $26, $4f, $27
  .byt $4f, $28, $4f, $29
InitialSFXEnd:
.endproc

.proc nmi
  pha
  inc retraces
  lda #2
  sta OAM_DMA
  pla
  rti
.endproc

.proc irq
  rti
.endproc

CHR:
.incbin "ascii.chr"
